// I2C Routinen fuer 80517 und SB8V6 Board
//	8.10.2005	mpa
#include<reg517a.h>
#include <I2C.h>

// Hardwareabgaengige Werte
sbit SDA = P3^4;							// SDA Leitung an P3.4
sbit SCL = P3^5;							// SDA Leitung an P3.5

#define 	I2CDELAY	10					// Verzoegerung auf I2C-Bus

#define 	H 				0x01			// Definition fuer logisch 1
#define		L 				0x00			// Definition fuer logisch 0

//------------------------------------------------------------------------------
// Routine i2c_init
// Purpose:	SDA,SCL in Ruhezustand versetzen
//------------------------------------------------------------------------------
void i2c_init(void)
{
SDA = H;									// Ruherzustand High
SCL = H;									// Ruherzustand High
}

//------------------------------------------------------------------------------
// 	Routine:	i2c_start
//	Purpose:	Start bit senden
//			   ____
//		SCL:       |__
//			   __	   
//		SDA:     |____
//	
//------------------------------------------------------------------------------
void i2c_start(void)
{


SDA	=	L;									// Datenleitung auf LOW
delay_time(I2CDELAY);						// Warte bis Signal eingeschwungen
SCL	=	L;									// Clockleitung auf LOW
delay_time(I2CDELAY);						// Warte bis Signal eingeschwungen	
}

//------------------------------------------------------------------------------
// 	Routine:	i2c_stop
//	Purpose:	Stop bit senden
//			     ____
//		SCL: ___|	
//			 	   __
//		SDA: _____|
//	
//------------------------------------------------------------------------------
void i2c_stop(void)
{
SCL	=	H;									// Clockleitung auf HIGH
delay_time(I2CDELAY);						// Warte bis Signal eingeschwungen
SDA	=	H;									// Datenleitung auf HIGH
delay_time(I2CDELAY);						// Warte bis Signal eingeschwungen
}
//********************************************************************************
// 	Routine:	i2c_write
//	parameter:	Datenbyte 	(inklusive R/W bit as D0 !)
//	Purpose:	Datenbyte schreiben
//	Returns:	ACK-Zustand
//
//			   _   _   _   _   _   _   _   _  	_
//		SCL: _|	|_|	|_|	|_|	|_|	|_|	|_|	|_|	|__| |____
//			 								  ________
//		SDA: DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
//	
//********************************************************************************
bit i2c_write(unsigned char output_data)
{
unsigned char index;


for(index = 0; index < 8; index++)  		// Sende 8 bits
	{                                		 
	if(output_data & 0x80) 	{SDA = 1;}		// Bestimme den Ausgangspegel durch Maskieren
	else					{SDA = 0;}
	delay_time(I2CDELAY);					// Warte bis Signal eingeschwungen
    output_data  <<= 1;            			// Ausgangsdaten ein bit nach links schieben
	SCL	=	H;								// Clockleitung auf HIGH
	delay_time(I2CDELAY);					// Warte bis Signal eingeschwungen
	SCL	=	L;								// Clockleitung auf LOW
	delay_time(I2CDELAY);					// Warte bis Signal eingeschwungen				
	}

SDA	=	H;									// Datenleitung auf HIGH
delay_time(I2CDELAY);						// Warte bis Signal eingeschwungen
SCL	=	H;									// Clockleitung auf HIGH
delay_time(I2CDELAY);						// Warte bis Signal eingeschwungen
if (SDA == 0)								// Lese  ACK bit
    {
	SCL	=	L;								// Clockleitung auf LOW
	delay_time(I2CDELAY);					// Warte bis Signal eingeschwungen
	return(1);								// ACK empfangen
	}
else
	{
	SCL	=	L;								// Clockleitung auf LOW
	delay_time(I2CDELAY);					// Warte bis Signal eingeschwungen
	return(0);								// Kein ACK empfangen
	}
}


//********************************************************************************
// 	Routine:	i2c_read
//	parameter:	send_ack	0...kein ACK senden, 1... ACK senden um Transfer zu beenden
//	Outputs:	Eingelesenes byte 
//	Purpose:	Datenbyte lesen
//
//			   _   _   _   _   _   _   _   _  	_
//		SCL: _|	|_|	|_|	|_|	|_|	|_|	|_|	|_|	|__| |____
//			 _________________________________		_
//		SDA:   								  |xxx_|
//			
//********************************************************************************
unsigned char i2c_read(char send_ack)
{
unsigned char index, input_data;

input_data = 0x00;



SDA	=	H;								// Datenleitung auf HIGH
delay_time(I2CDELAY);					// Warte bis Signal eingeschwungen	

for(index = 0; index < 8; index++)  	// 8 bit einlesen MSB zuerst
	{
    input_data <<= 1;					// Eingabedaten um  eins nach links schieben
	SCL	=	H;							// Clockleitung auf HIGH
	delay_time(I2CDELAY);				// Warte bis Signal eingeschwungen
	input_data = input_data | SDA;		// Datenbit lesen und zu Inputdata zufuegen
	SCL	=	L;							// Clockleitung auf LOW
	delay_time(I2CDELAY);				// Warte bis Signal eingeschwungen							
	}
	

if (send_ack) 	SDA = L;				// SDA auf LOW falls ACK gefordert
else 			SDA = H;				// ansonsten bleibet SDA auf HIGH
delay_time(I2CDELAY);					// Warte bis Signal eingeschwungen	
SCL	=	H;								// Clockleitung auf HIGH
delay_time(I2CDELAY);					// Warte bis Signal eingeschwungen
SCL	=	L;								// Clockleitung auf LOW
delay_time(I2CDELAY);					// Warte bis Signal eingeschwungen	
SDA	=	L;								// Datenleitung auf LOW
delay_time(I2CDELAY);					// Warte bis Signal eingeschwungen


return(input_data);
}

//-------------------------------------------------------------------------------
// Schreibe Daten in Register
//	Parameter:	
//				I2C_Addr...Slave Adresse des Bausteins
//				Reg_Addr...Register in das geschrieben werden soll
//				Reg_Data...Daten, die ins Register geschrieben werden sollen
//-------------------------------------------------------------------------------
void I2C_Schreiben(unsigned char I2C_Addr, unsigned char Reg_Addr, unsigned char Reg_Data)
{
i2c_start();					// Startbit senden
i2c_write(I2C_Addr);			// Slave Adresse, schreiben
i2c_write(Reg_Addr);			// Register Adresse schreiben
i2c_write(Reg_Data);			// Register Daten schreiben
i2c_stop();						// Stopbit senden
}

//-------------------------------------------------------------------------------
// Lese Daten von Register
//	Parameter:	
//				I2C_Addr...Slave Adresse des Bausteins
//				Reg_Addr...Register aus dem gelesen werden soll
//	Return:		Reg_Data...Daten, die aus Register gelesen wurden
//-------------------------------------------------------------------------------
unsigned char  I2C_Lesen(unsigned char I2C_Addr, unsigned char Reg_Addr)
{
unsigned char Daten;
i2c_start();					// Startbit senden
i2c_write(I2C_Addr);			// Slave Adresse, schreiben
i2c_write(Reg_Addr);			// Register Adresse schreiben
i2c_stop();						// Stopbit senden
i2c_start();					// Startbit senden
I2C_Addr = I2C_Addr | 0x01;		// LSB setzten zum Lesen
i2c_write(I2C_Addr);			// Slave Adresse, lesen
Daten = i2c_read(0);			// Daten lesen und kein ACK schicken => Transfer ende
i2c_stop();						// Stopbit senden
return(Daten);					// Datenwert zurueckgeben
}

//********************************************************************************
// 	Routine:	delay_time
// parameter:	delay...0-255 Delay time 
//********************************************************************************
void delay_time(unsigned char delay)
{
unsigned char i;
for(i=0;i<delay;i++)
	{	;	}
}